import { App, ComponentInternalInstance, InjectionKey } from 'vue';
import { EventsBackboneSpine } from '../plugins/event-backbone.ts';
import { default as plugin } from '../index';
declare module "vue" {
    interface EventsBackboneProperties {
        $eventsBackbone: {
            eventsBackboneSpine: EventsBackboneSpine;
        };
    }
}
export default plugin;
export * from '../index';
export interface EventsBackboneSpineInterface {
    checkKeys(obj: any): number;
    emitEvent(currentInstance: ComponentInternalInstance, ev: string, data?: any, config?: {
        global?: boolean;
        eager?: boolean;
    }): void;
    on(componentInstance: ComponentInternalInstance, ev: string, h: EventsBackboneEventHandler, opts?: EventsBackboneSpineEntryOption): void;
    off(uid: number, ev: string, h?: EventsBackboneEventHandler): void;
    offAll(uid: number): void;
}
export interface installEventsBackbone {
    install: (app: App) => void;
}
export type EventsBackboneEventHandler = ((be: EventsBackboneSpineEvent) => void);
export interface EventsBackboneSpineEvent {
    emitterComponentInstance: ComponentInternalInstance;
    handlerCallerComponentInstance: ComponentInternalInstance;
    eventName: string;
    eventData?: any;
    global?: boolean;
    propagationStopped?: boolean | null;
    eager?: boolean | null;
    stopPropagation: (() => void);
    once: (() => void);
}
export interface EventsBackboneSpineEntryOption {
    stopPropagation?: boolean | ((backboneEvent: EventsBackboneSpineEvent) => boolean);
    once?: boolean | ((backboneEvent: EventsBackboneSpineEvent) => boolean);
}
export type EventsBackboneSpineEntryOptions = {
    [handlerName: string]: EventsBackboneSpineEntryOption;
};
export interface EventsBackboneSpineEntry {
    uid: number;
    registeredHandlers: {
        [eventName: string]: Set<EventsBackboneEventHandler>;
    };
}
export type EventsBackboneDirectiveParam = {
    handler: EventsBackboneEventHandler;
    options?: EventsBackboneSpineEntryOption;
};
export type EventsBackboneDirectiveParams = {
    [key: string]: Array<EventsBackboneDirectiveParam>;
};
export type EventsBackboneEmitter = (<T>(data?: T, global?: boolean, eager?: boolean) => Promise<void>);
export type EventsBackboneEmitterGenerator = (evt: string) => EventsBackboneEmitter;
export declare const createEventsBackboneEmitter: InjectionKey<EventsBackboneEmitterGenerator>;
